@@ -190,12 +190,15 @@ class Connection(abstract.FileDescriptor, tcp._AbortingMixin):
 
         @see: L{twisted.internet.interfaces.IWriteDescriptor.doWrite}.
         """
-
-        # Send as much data as you can.
+        # If we're empty return
         if len(self.messageBuffer) == 0:
+            #check for graceful disconnection
+            if self.disconnecting:
+                return self._postLoseConnection()
             return None
+        # Send as much data as you can.
         result = self.splitAndSend()
-        if len(self.messageBuffer) == 0:
+        if len(self.messageBuffer) == 0 or result:
             #self.stopWriting()
             #Really stop writable event
             self.reactor._poller.remove_usock(self.socket.UDTSOCKET)
